//=============================================================================
// Copyright (C) 2023, Qorvo, Inc.
//
// THIS SOFTWARE IS SUBJECT TO A SOURCE CODE LICENSE AGREEMENT WHICH PROVIDES,
// AMONG OTHER THINGS:  (i) THAT IT CAN BE USED ONLY TO ADAPT THE LICENSEE'S
// APPLICATION TO PAC PROCESSORS SUPPLIED BY QORVO, INC.;
// (ii) THAT  IT IS PROVIDED "AS IS" WITHOUT WARRANTY;  (iii) THAT
// QORVO, INC. IS NOT LIABLE FOR ANY INDIRECT DAMAGES OR FOR DIRECT
// DAMAGES EXCEEDING US$1,500;  AND (iv) THAT IT CAN BE DISCLOSED TO AND USED
// ONLY BY CERTAIN AUTHORIZED PERSONS.
//
//=============================================================================

#include "app.h"

//=====================================
/// @select one detail function for test
//=====================================
#define TEST_MANUAL_ADC
// #define TEST_ADC_DTSE_SEQUENCE_EXAMPLE1
// #define TEST_ADC_DTSE_MULTI_SEQUENCE

//========================================================================================================================
// There are two ways to use the ADC in the device. Manual and DTSE
// The DTSE is the recommended way to sample the ADC for most applications
// In this app, all ADC inputs are sampled(including set AIO pins as ADC input) and sent over UART.
//      Manual: various channels are sampled by setting the correct register and starting the ADC, then the converted value can be read
//      DTSE: The DTSE is initilaized for various conversions.  When interrupts occur during a DTSE sequence, the 
//            interrupt handlers in adc_isr.c load the sampled values so they can be written over the UART
//========================================================================================================================
void adc_app(void)
{
    // Disable global interrupts until initialization is complete
    __disable_irq();

    system_init();
    uart_init(UARTC, 115200);

    GPIO_output();

    pac_tile_socbridge_config(1, 0);                // Configure SOC Bridge for talking to CAFE
    configurable_analog_front_end_init();           // init the cafe registers

    // Initialize optional ADC Gain and Offset that improve ADC accuracy; note that these are only used in a few examples
    init_float_adc_gain_and_offset();
    init_fixed_point_adc_gain_and_offset();

    #ifdef TEST_MANUAL_ADC
        test_manual_adc();
    #endif

    #ifdef TEST_ADC_DTSE_SEQUENCE_EXAMPLE1
        test_adc_dtse_sequence_example_1();
    #endif

    #ifdef TEST_ADC_DTSE_MULTI_SEQUENCE
        test_adc_dtse_multi_sequence();
    #endif

}

void test_manual_adc(void)
{
    adc_manual_init();                              // init the ADC to manual mode

    while (1)
    {
        delay_times(500000);
        printf("\r\n---------------ADC Manual TEST---------------\r\n");
        adc_manual_sample();                        // Convert different channels
        adc_manual_convert_vcore();                 // Convert the VCORE voltage
        adc_manual_convert_vcc33();                 // Convert the VCC33 voltage
        adc_manual_convert_vccio();                 // Convert the VCCIO voltage
        adc_manual_convert_vsys();                  // Convert the VSYS voltage
        adc_manual_convert_vp();                    // Convert the VP voltage
        adc_manual_convert_vref();                  // Convert the VREF voltage
        delay_times(5000);
        adc_manual_convert_temperature();           // Convert and calculate device Temperature
        adc_manual_convert_vcore_calibrated();      // Convert Vcore using INFO FLASH ADCOFFSET and ADCGAIN
    }
}

void test_adc_dtse_sequence_example_1(void)
{
    configure_timer_c_pwm_trigger_adc();            // init a PWM to trigger the ADC
    init_adc_dtse_sequence_example_1();             // init the ADC in DTSE mode

    __enable_irq();

    while (1)
    {
        delay_times(500000);

        // The ADC values will be auto loaded in interrupt adc_isr.c . here just send back the values
        printf("\n\n");
        printf("\r\n--------------- DTSE Sequence test ---------------\r\n");
        printf("ADC4 (PF4) = %4d \n", adc_val[0 + SEND_AFTER_CONVERSION]);
        printf("ADC5 (PF5) = %4d \n", adc_val[1 + SEND_AFTER_CONVERSION]);
        printf("ADC6 (PF6) = %4d \n", adc_val[2 + SEND_AFTER_CONVERSION]);
        printf("VCORE      = %4d \n", adc_val[3 + SEND_AFTER_CONVERSION]);
        printf("VCORE*4/10 = %4d \n", adc_val[4 + SEND_AFTER_CONVERSION]);
        printf("VDDA*4/10  = %4d \n", adc_val[5 + SEND_AFTER_CONVERSION]);
        printf("VCCIO*4/10 = %4d \n", adc_val[6 + SEND_AFTER_CONVERSION]);
        printf("VSYS*4/10  = %4d \n", adc_val[7 + SEND_AFTER_CONVERSION]);
        printf("VPTAT      = %4d \n", adc_val[8 + SEND_AFTER_CONVERSION]);
        printf("VP/10      = %4d \n", adc_val[9 + SEND_AFTER_CONVERSION]);
    }
}

void test_adc_dtse_multi_sequence(void)
{
    configure_timer_c_pwm_trigger_adc();            // init a PWM to trigger the ADC
    init_adc_dtse_multi_sequence();                 // init the ADC in DTSE mode

    __enable_irq();

    while (1)
    {
        delay_times(500000);

        printf("\n\n");
        printf("\r--------------- Multi DTSE Sequence test ---------------\n");
        
        printf("\r\nDTSE Trigger 21 - Timer C, CCR5\n");
        printf("VSYS*4/10  = %4d \n", adc_val[0 + SEND_AFTER_CONVERSION]);
        // printf("VSYS*4/10  = %4d \n", adc_val[1 + SEND_AFTER_CONVERSION]);      // dummy
        printf("VP/10      = %4d \n", adc_val[2 + SEND_AFTER_CONVERSION]);
        // printf("VP/10      = %4d \n", adc_val[3 + SEND_AFTER_CONVERSION]);      // dummy
        printf("VREF/2     = %4d \n", adc_val[4 + SEND_AFTER_CONVERSION]);
        // printf("VREF/2     = %4d \n", adc_val[5 + SEND_AFTER_CONVERSION]);      // dummy   
        printf("VPTAT      = %4d \n", adc_val[6 + SEND_AFTER_CONVERSION]);
        // printf("VPTAT      = %4d \n", adc_val[7 + SEND_AFTER_CONVERSION]);      // dummy
        printf("VCORE      = %4d \n", adc_val[8 + SEND_AFTER_CONVERSION]);
        // printf("VCORE      = %4d \n", adc_val[9 + SEND_AFTER_CONVERSION]);      // dummy
        printf("ADC0       = %4d \n", adc_val[10 + SEND_AFTER_CONVERSION]);

        printf("\r\nDTSE Trigger 22 - Timer C, CCR6\n");
        printf("ADC5 (PF5) = %4d \n", adc_val[11 + SEND_AFTER_CONVERSION]);
        printf("ADC6 (PF6) = %4d \n", adc_val[12 + SEND_AFTER_CONVERSION]);
        printf("ADC7 (PF7) = %4d \n", adc_val[13 + SEND_AFTER_CONVERSION]);

        printf("\r\nDTSE Trigger 23 - Timer C, CCR7\n");
        printf("VCORE*4/10 = %4d \n", adc_val[14 + SEND_AFTER_CONVERSION]);
        // printf("VCORE*4/10 = %4d \n", adc_val[15 + SEND_AFTER_CONVERSION]);      // dummy
        printf("VDDA*4/10  = %4d \n", adc_val[16 + SEND_AFTER_CONVERSION]);
        // printf("VDDA*4/10  = %4d \n", adc_val[17 + SEND_AFTER_CONVERSION]);      // dummy
        printf("VCCIO*4/10 = %4d \n", adc_val[18 + SEND_AFTER_CONVERSION]);
        // printf("VCCIO*4/10 = %4d \n", adc_val[19 + SEND_AFTER_CONVERSION]);      // dummy
        printf("VCORE      = %4d \n", adc_val[20 + SEND_AFTER_CONVERSION]);
        printf("VREF/2     = %4d \n", adc_val[21 + SEND_AFTER_CONVERSION]);

        printf("ADC4 (PF4) = %4d \n", adc_val[22 + SEND_AFTER_CONVERSION]);
        printf("ADC0       = %4d \n", adc_val[0]);
    }
}
