//=============================================================================
// Copyright (C) 2023, Qorvo, Inc.
//
// THIS SOFTWARE IS SUBJECT TO A SOURCE CODE LICENSE AGREEMENT WHICH PROVIDES,
// AMONG OTHER THINGS:  (i) THAT IT CAN BE USED ONLY TO ADAPT THE LICENSEE'S
// APPLICATION TO PAC PROCESSORS SUPPLIED BY QORVO, INC.;
// (ii) THAT  IT IS PROVIDED "AS IS" WITHOUT WARRANTY;  (iii) THAT
// QORVO, INC. IS NOT LIABLE FOR ANY INDIRECT DAMAGES OR FOR DIRECT
// DAMAGES EXCEEDING US$1,500;  AND (iv) THAT IT CAN BE DISCLOSED TO AND USED
// ONLY BY CERTAIN AUTHORIZED PERSONS.
//
//=============================================================================

#include "app.h"

#define TEMPS_ADDR      0x0010041C   
#define VREF_ADDR       0x00100436
#define KELVIN_TO_CELSIUS   273.15f

void configurable_analog_front_end_init(void)
{

#ifdef CAFE_ARCH1
    static volatile unsigned char reg_value;
    
    // If any power manager error bits set on startup, clear them
    if (pac_tile_register_read(ADDR_PWRFAULT))
        pac_tile_register_write(ADDR_PWRFAULT, 0xFF);

    // Reset AFE registers to default state
    pac_tile_register_write(ADDR_AFEMUXCTL, 0);
    pac_tile_register_write(ADDR_DRVCTL, 0);
    pac_tile_register_write(ADDR_TEMPFAULT, 0);
    pac_tile_register_write(ADDR_AFECTL1, 0);

    pac_tile_register_write(ADDR_AFECTL1, 0x41);                             // Set MCUALIVE and Enable signal manager
    reg_value = pac_tile_register_read(ADDR_AFECTL1);
    // Enable mux scan mode
    pac_tile_register_write(ADDR_AFEMUXCTL, 0x02);                            // BUFFEN=1, EMUX_EN=1, for auto-scan, sample and hold AO10SH=1, AO32SH=1, AO54SH=1

    (void) reg_value;      // Added to prevent compiler warnings about the variable being set but not unused

#endif

}

void adc_manual_init(void)
{
    // EMUX configuration
    pac_adc_config_emux_io();

    // Configure ADC input pin PG7=ADO
    pac_adc_config_io(1);
    pac_adc_config(ADCCTL_MODE_MANUAL, ADCCTL_CLKDIV_DIV8, 0);

    PAC_ADC->ADCCTL.INTENMAN = IRQ_DIS;                                     //ADC Manual Mode Conversion Complete Interrupt
    PAC_ADC->ADCCTL.INTENEMUX = IRQ_DIS;                                    //EMUX Complete Interrupt
    PAC_ADC->ADCCTL.START = 1;                                              //Start ADC Conversion
    PAC_ADC->ADCCTL.ENABLE = 1;                                             //ADC module Enable

    // Select PF4(AD4) for ADC Input form AFE
    PAC_GPIOF->MODE.P4 = 0;                                                 // ADC4 INPUT
    PAC_SCC->PFMUXSEL.P4 = 0;                                               // Set Port F Pin 4 ADCINPUT

    // Select PF5(AD5) for ADC Input form AFE
    PAC_GPIOF->MODE.P5 = 0;                                                 // ADC5 INPUT
    PAC_SCC->PFMUXSEL.P5 = 0;                                               // Set Port F Pin 5 ADCINPUT

    // Select PF6(AD6) for ADC Input form AFE
    PAC_GPIOF->MODE.P6 = 0;                                                 // ADC6 INPUT
    PAC_SCC->PFMUXSEL.P6 = 0;                                               // Set Port F Pin 6 ADCINPUT

    // Select PF7(AD7) for ADC Input form AFE
    PAC_GPIOF->MODE.P7 = 0;                                                 // ADC7 INPUT
    PAC_SCC->PFMUXSEL.P7 = 0;                                               // Set Port F Pin 7 ADCINPUT

    //Enable ADC
    pac_adc_enable(1);                                                      // Enable ADC. Automated sequences will begin after issuing ADC start in main()
    // Start ADC sequencer
    pac_adc_start();
}

//==============================================================================
///
/// @brief  This function performs the ADC conversion and returns the result.
///
/// @return 12 bit ADC output - a right justified 16bit value:
///         0x0000 = 0V and 0x0FFF = 2.5V
//==============================================================================
uint16_t adc_convert(void)
{

    uint16_t result;

    //===== Start ADC Conversion =====
    PAC_ADC->ADCCTL.START = 1;

    //===== Wait for ADC Conversion to Finish =====
    while (PAC_ADC->ADCCTL.ADBUSY) { }

    //===== Get ADC Result =====
    result = PAC_ADC->ADCRES.VAL;

    return result;
}

// Global Variables containing ADC Gain and Offset for calibrating ADC conversions
float adc_gain;       // ADC Gain in counts/counts
float adc_offset;     // ADC Offset in counts
//==============================================================================
///
/// @brief  This function computes the floating point ADC Gain and Offset for
///         calibrating ADC results but keeping it in counts.   It uses the ADCGAIN
///         and ADCOFFSET values stored in INFO2
///
///         INFO2 ADCGAIN is stored as counts/V * 2^16
///         INFO2 ADCOFFSET is stored as counts * 2^16 (Two's complement)
///         These are used in the equation
///         Yc = m * Xv + b ; Yc in counts, Xv in volts
///         Xv in Volts is the input voltage to the ADC
///         Yc is the ADC output in counts
///         m = (ADCGAIN / 2^16)  counts/V
///         b = (ADCOFFSET / 2^16) counts
///
/// @return None
///
//==============================================================================
void init_float_adc_gain_and_offset(void)
{
    //===== Caclulate ADC Offset and Gain using INFO FLASH contents=====
    // We begin with the slope/intercept equation
    // Y = mX + b
    // X in volts
    // Y in counts
    // m = ADCGAIN / 2^16; note that ADCGAIN is stored as counts/V * 2^16
    // b = ADCOFFSET / 2^16; note that ADCOFFSET is stored as counts * 2^16 (Two's Complement)
    // Or written as
    // Yc = m * Xv + b ; Yc in counts, Xv in volts
    //
    // If we rearange the equation to go from ADC output in counts to Volts we get
    // Xv = 1/m(Yc - b) ;  Xv in Volts   Yc in Counts
    // Puting the above equation in the form of Y = mX + b we have
    // Yv = m' * Xc + b' ; where m' = 1/m and b' = -(1/m)*b
    // Xc is the ADC result in counts and Yv is the result in Volts
    //
    // If we want to keep the output in ADC counts, then we need to scale by counts/volt
    // So, we have
    // Yc = (4095 counts/ 2.5 volts) * (m' * Xc + b')
    // or
    // Yc = m'' * Xc + b''  ; where m'' = (1/m)*(4095/2.5) and b' = -(1/m)*b*(4095/2.5)
    //
    // ===== Finally we have =====
    // Yc = m'' * Xc + b''  ; where m'' = (4095/2.5)/m and b' = -(4095/2.5)/m)*b  = -m'' * b
    // Xc is the ADC result in counts and Yc is the calibrated result in counts
    //
    // Below we calculate
    // m'' = adc_gain in counts/counts
    // b'' = adc_offset in counts
    // This only needs to be calculated once at init time.

    // Compute floating point equivalent of adc_gain from INFO FLASH
    adc_gain = (int32_t)PAC_INFO2->ADCGAIN;
    adc_gain = adc_gain / 65536.0f;

    // Compute m'' = adc_gain
    adc_gain = (4095.0f / 2.5f) / adc_gain;           // m'' = (4095/2.5) / m


    // Compute floating point equivalent of adc_offset from INFO FLASH
    adc_offset = (int32_t)PAC_INFO2->ADCOFFSET;
    adc_offset = adc_offset / 65536.0f;

    // Compute b'' = adc_offset = -m'' * b
    adc_offset = -adc_gain * adc_offset;
}

//==============================================================================
///
/// @brief  This function performs the ADC conversion and returns the result in
///         counts
///         It calibrates the results using adc_gain and adc_offset computed from the
///         ADCGAIN and ADCOFFSET stored in INFO2
///
/// @return Calibrated ADC result in ADC counts
///         0x0000 = 0V and 0x0FFF = 2.5V
//==============================================================================
uint16_t adc_convert_float_calibrated(void)
{

    float result;
    float result_calibrated;
    int32_t integer_result_counts;

    //===== Start ADC Conversion =====
    PAC_ADC->ADCCTL.START = 1;

    //===== Wait for ADC Conversion to Finish =====
    while (PAC_ADC->ADCCTL.ADBUSY) { }

    //===== Get ADC Result =====
    result = PAC_ADC->ADCRES.VAL;

    // Calibrate results using device specific Gain and Offset;  y = mx + b
    result_calibrated = adc_gain * result + adc_offset;

    // Result must be positive
    if (result_calibrated < 0.0f)
        result_calibrated = 0.0;

    integer_result_counts = (int)result_calibrated;

    return integer_result_counts;
}


#define Q20_4095_DIV_2P5 ((uint32_t)(4095.0/2.5 * 1048576))      // 1.0 * 2^20

// Global Variables containing ADC Gain and Offset for calibrating ADC conversions
int32_t adc_gain_q14;                                               // Q14 ADC Gain in volts/count
int32_t adc_offset_q14;                                             // Q14 ADC Offset in volts
//==============================================================================
///
/// @brief  This function computes the fixed point Q14 ADC Gain and Offset for
///         calibrating ADC results.   It uses the ADCGAIN and ADCOFFSET values
///         stored in INFO2
///
///         INFO2 ADCGAIN is stored as counts/V * 2^16
///         INFO2 ADCOFFSET is stored as counts * 2^16 (Two's complement)
///         These are used in the equation
///         Yc = m * Xv + b ; Yc in counts, Xv in volts
///         Xv in Volts is the input voltage to the ADC
///         Yc is the ADC output in counts
///         m = ADCGAIN / 2^16  counts/V
///         b = ADCOFFSET / 2^16 counts
///
/// @return None
///
//==============================================================================
void init_fixed_point_adc_gain_and_offset(void)
{
    //===== Caclulate ADC Offset and Gain using INFO FLASH contents=====
    // We begin with the slope/intercept equation
    // Y = mX + b
    // X in volts
    // Y in counts
    // m = ADCGAIN / 2^16; note that ADCGAIN is stored as counts/V * 2^16
    // b = ADCOFFSET / 2^16; note that ADCOFFSET is stored as counts * 2^16 (Two's Complement)
    // Or written as
    // Yc = m * Xv + b ; Yc in counts, Xv in volts
    //
    // If we rearange the equation to go from ADC output in counts to Volts we get
    // Xv = 1/m(Yc - b) ;  Xv in Volts   Yc in Counts
    // Puting the above equation in the form of Y = mX + b we have
    // Yv = m' * Xc + b' ; where m' = 1/m and b' = -(1/m)*b
    // Xc is the ADC result in counts and Yv is the result in Volts
    //
    // If we want to keep the output in ADC counts, then we need to scale the gain by counts/volt
    // So, we have
    // Yc = (4095 counts/ 2.5 volts) * (m' * Xc + b')
    // or
    // Yc = m'' * Xc + b''  ; where m'' = (1/m)*(4095/2.5) and b' = -(1/m)*b*(4095/2.5)
    //
    // ===== Finally we have =====
    // Yc = m'' * Xc + b''  ; where m'' = (4095/2.5)/m and b' = -(4095/2.5)/m)*b  = -m'' * b
    // Xc is the ADC result in counts and Yc is the calibrated result in counts
    //
    // Below we calculate
    // m'' = adc_gain in counts/counts
    // b'' = adc_offset in counts
    // This only needs to be calculated once at init time.

    adc_gain_q14 = (Q20_4095_DIV_2P5 / ((int32_t)PAC_INFO2->ADCGAIN >> 6)) << 4;       // Q14 Gain in volts/count,   Q20/(Q16 >> 6) = Q10, Q10<<4 = Q14

    adc_offset_q14 = (-((int32_t)PAC_INFO2->ADCOFFSET>>6) * adc_gain_q14) >> 10;       // Q14 Offset in counts,   (Q16>>6 * Q14) = Q24,   Q24 >> 10 = Q14
}

//==============================================================================
///
/// @brief  This function performs the ADC conversion and returns the result in
///         counts
///         It calibrates the results using adc_gain_q14 and adc_offset_q14 computed from the
///         ADCGAIN and ADCOFFSET stored in INFO2
///
/// @return Calibrated ADC result in ADC counts
///         0x0000 = 0V and 0x0FFF = 2.5V
//==============================================================================
uint32_t adc_convert_fixed_point_calibrated(void)
{

    uint16_t result;
    int32_t result_calibrated;

    //===== Start ADC Conversion =====
    PAC_ADC->ADCCTL.START = 1;

    //===== Wait for ADC Conversion to Finish =====
    while (PAC_ADC->ADCCTL.ADBUSY) { }

    //===== Get ADC Result =====
    result = PAC_ADC->ADCRES.VAL;

    //===== Calibrate results using device specific Gain and Offset;  y = mx + b
    result_calibrated = ((result * adc_gain_q14) + adc_offset_q14) >> 14;     // ((Q0 * Q14) + Q14) = Q14 then shift right by 14 = Q0
    if (result_calibrated < 0)
        result_calibrated = 0;

    return (uint16_t)result_calibrated;
}


void adc_manual_convert_vcore_calibrated(void)
{
    int16_t adc_value;
    int16_t adc_value_fixed_point_cal;
    int16_t adc_value_float_cal;
    float voltage;
    float voltage_scale_factor;

    //  Select ADC Channel ADC0
    PAC_ADC->ADCCTL.CHANNEL = ADCCTL_CHANNEL_ADC0;

#ifdef CAFE_ARCH1
    // AFE Mux Select
    pac_tile_register_write(ADDR_AFEMUXSEL, AFEMUXSEL_AFEMUX_VCORE);        // VCORE

    voltage_scale_factor = 1.0;

#endif

    // Perform ADC Conversion without Calibration
    adc_value = adc_convert();
    voltage = adc_value  * (2.5 / 4095) * voltage_scale_factor;                             // counts * (2.5V / 4095 counts)
    printf("VCORE no Calibration = %x counts = %0.3fV \n", adc_value, voltage);

    // Perform ADC Conversion with floating point Calibration
    adc_value_float_cal = adc_convert_float_calibrated();
    voltage = adc_value_float_cal  * (2.5 / 4095) * voltage_scale_factor;                   // counts * (2.5V / 4095 counts)
    printf("VCORE Float Calibrated = %x counts = %0.3fV \n", adc_value_float_cal, voltage);

    // Perform ADC Conversion with fixed point Calibration
    adc_value_fixed_point_cal = adc_convert_fixed_point_calibrated();
    voltage = adc_value_fixed_point_cal * (2.5 / 4095) * voltage_scale_factor;              // counts * (2.5V / 4095 counts)
    printf("VCORE Fixed Point Calibrated = %x counts = %0.3fV \n", adc_value_fixed_point_cal, voltage);

}

void adc_manual_convert_vcore(void)
{
    int16_t adc_value;
    float voltage;
    float voltage_scale_factor;

    //  Select ADC Channel ADC0
    PAC_ADC->ADCCTL.CHANNEL = ADCCTL_CHANNEL_ADC0;

#ifdef CAFE_ARCH1
    // AFE Mux Select
    pac_tile_register_write(ADDR_AFEMUXSEL, AFEMUXSEL_AFEMUX_VCORE);        // VCORE
      
    voltage_scale_factor = 1.0;

#endif

    // Perform ADC Conversion
    adc_value = adc_convert();

    voltage = adc_value  * (2.5f / 4095) * voltage_scale_factor;                     // counts * (2.5V / 4095 counts)
   	printf("\nVCORE = %0.2fV \n", voltage);
}


void adc_manual_convert_vcc33(void)
{
    int16_t adc_value;
    float voltage;
    float voltage_scale_factor;

    //  Select ADC Channel ADC0
    PAC_ADC->ADCCTL.CHANNEL = ADCCTL_CHANNEL_ADC0;

#ifdef CAFE_ARCH1
    // AFE Mux Select
    pac_tile_register_write(ADDR_AFEMUXSEL, AFEMUXSEL_AFEMUX_VDDA_DIV2P5);  // VDDA * 4 / 10
    
    voltage_scale_factor = 10.0 / 4.0;

#endif

    // Perform ADC Conversion
    adc_value = adc_convert();

    voltage = adc_value  * (2.5f / 4095) * voltage_scale_factor;                   // counts * (2.5V / 4095 counts) * scale factor
    printf("VDDA = %0.2fV \n", voltage);
}


void adc_manual_convert_vccio(void)
{
    int16_t adc_value;
    float voltage;
    float voltage_scale_factor;

    //  Select ADC Channel ADC0
    PAC_ADC->ADCCTL.CHANNEL = ADCCTL_CHANNEL_ADC0;

#ifdef CAFE_ARCH1
    // AFE Mux Select
    pac_tile_register_write(ADDR_AFEMUXSEL, AFEMUXSEL_AFEMUX_VCCIO_DIV2P5); // VCCIO * 4 / 10
    
    voltage_scale_factor = 10.0 / 4.0;

#endif

    // Perform ADC Conversion
    adc_value = adc_convert();

    voltage = adc_value  * (2.5f / 4095) * voltage_scale_factor;                     // counts * (2.5V / 4095 counts)
    printf("VCCIO = %0.2fV \n", voltage);
}


void adc_manual_convert_vsys(void)
{
    int16_t adc_value;
    float voltage;
    float voltage_scale_factor;

    //  Select ADC Channel ADC0
    PAC_ADC->ADCCTL.CHANNEL = ADCCTL_CHANNEL_ADC0;

#ifdef CAFE_ARCH1
    // AFE Mux Select
    pac_tile_register_write(ADDR_AFEMUXSEL, AFEMUXSEL_AFEMUX_VSYS_DIV2P5);  // VSYS * 4 / 10
    
    voltage_scale_factor = 10.0 / 4.0;

#endif

    // Perform ADC Conversion
    adc_value = adc_convert();

    voltage = adc_value  * (2.5f / 4095) * voltage_scale_factor;                     // counts * (2.5V / 4095 counts)
    printf("VSYS = %0.2fV \n", voltage);
}


void adc_manual_convert_vp(void)
{
    int16_t adc_value;
    float voltage;
    float voltage_scale_factor;

    //  Select ADC Channel ADC0
    PAC_ADC->ADCCTL.CHANNEL = ADCCTL_CHANNEL_ADC0;

#ifdef CAFE_ARCH1
    // AFE Mux Select
    pac_tile_register_write(ADDR_AFEMUXSEL, AFEMUXSEL_AFEMUX_VP_DIV10);     // VP / 10
    
    voltage_scale_factor = 10.0 / 1.0;

#endif

    // Perform ADC Conversion
    adc_value = adc_convert();

    voltage = adc_value  * (2.5f / 4095) * voltage_scale_factor;                     // counts * (2.5V / 4095 counts)
    printf("VP = %0.2fV \n", voltage);
}

void adc_manual_convert_vref(void)
{
    int16_t adc_value;
    float voltage;
    float voltage_scale_factor;

    //  Select ADC Channel ADC0
    PAC_ADC->ADCCTL.CHANNEL = ADCCTL_CHANNEL_ADC0;

    // AFE Mux Select
    pac_tile_register_write(ADDR_AFEMUXSEL, AFEMUXSEL_AFEMUX_VREF_DIV2);    // VREF / 2
    
    voltage_scale_factor = 2.0 / 1.0;

    // Perform ADC Conversion
    adc_value = adc_convert();

    voltage = adc_value  * (2.5f / 4095) * voltage_scale_factor;                     // counts * (2.5V / 4095 counts)
    printf("VREF = %0.2fV \n", voltage);
}

void adc_manual_convert_temperature(void)
{
    int16_t adc_value;

    //  Select ADC Channel ADC0
    PAC_ADC->ADCCTL.CHANNEL = ADCCTL_CHANNEL_ADC0;

#ifdef CAFE_ARCH1
    //================== Temperature Calculation =====================
    // VPTAT = (2.5 * ADC_CODE) / 4095
    // VT300K = TTEMPS * VREFCODE/10000 / 4095
    //
    // Temp Kelvin = 300 * (VPTAT + 0.075) / (VT300K + 0.075)
    // Temp Celsius = 300 * (VPTAT + 0.075) / (VT300K + 0.075) - 273.15
    //=================================================================
    float VT300K, VPTAT;
    static float TempK, TempC = 0;
    uint16_t TTEMPS, VREFCODE;

    TTEMPS = READ_UINT16(TEMPS_ADDR);           // Read InfoROM Temp sensor code
    VREFCODE = READ_UINT16(VREF_ADDR);          // Read InfoROM VREF code

    // AFE Mux Select
    pac_tile_register_write(ADDR_AFEMUXSEL, AFEMUXSEL_AFEMUX_VPTAT);        // VPTAT MUX
    
    // Perform ADC Conversion
    adc_value = adc_convert();

    VPTAT = adc_value * (2.5f / 4095);
    printf("\r\nVPTAT = %0.2fV \n", VPTAT);
		
    if( TTEMPS != 0xFFFF )                      // There is no Temp sensor code ? 
        VT300K = TTEMPS * (VREFCODE/10000.0) / 4095;
    else
        VT300K = 1.33f;

    // Temperature calculate
    TempK = 300 * (VPTAT+0.075f) / (VT300K+0.075f);
    TempC = (TempK - KELVIN_TO_CELSIUS);
    printf("Internal Temperature:  %0.2fC  \r\n", TempC);

#endif

}

uint32_t adc_manual_convert_channel(uint8_t adc_channel)
{
    uint16_t adc_value;

    //  Select ADC Channel ADC0
    PAC_ADC->ADCCTL.CHANNEL = adc_channel;

    // Perform ADC Conversion
    adc_value = adc_convert();

    return adc_value;
}

void adc_manual_sample(void)
{
    uint16_t adc_value;

    // Convert ADC4 input on PF4
    adc_value = adc_manual_convert_channel(ADCCTL_CHANNEL_ADC4);
    printf("ADC4 (PF4) = %4d\n", adc_value);

    // Convert ADC5 input on PF5
    adc_value = adc_manual_convert_channel(ADCCTL_CHANNEL_ADC5);
    printf("ADC5 (PF5) = %4d\n", adc_value);

    // Convert ADC6 input on PF6
    adc_value = adc_manual_convert_channel(ADCCTL_CHANNEL_ADC6);
    printf("ADC6 (PF6) = %4d\n", adc_value);

    // Convert ADC7 input on PF7
    adc_value = adc_manual_convert_channel(ADCCTL_CHANNEL_ADC7);
    printf("ADC7 (PF7) = %4d\n", adc_value);

}

void init_adc_dtse_sequence_example_1(void)
{
    // EMUX configuration
    pac_adc_config_emux_io();

    // Enable mux scan mode
    pac_tile_register_write(ADDR_AFEMUXCTL, 0x03);                            // BUFFEN=1, EMUX_EN=1, for auto-scan, sample and hold AO10SH=1, AO32SH=1, AO54SH=1

    // Configure ADC input pin PG7=ADO
    pac_adc_config_io(1);

    pac_adc_emux_config(ADCEMUXCTL_DTSE_SEQ, ADCEMUXCTL_EMUXDIV_DIV16);
    pac_adc_config(ADCCTL_MODE_DTSE, ADCCTL_CLKDIV_DIV16, 1);

    // Sequencer Configuration
    config_adc_dtse_sequence_example_1();

    // Set ADC interrupt priority and enable ADC interrupt
    NVIC_EnableIRQ(ADC0_IRQn);                                                  // Enable ADC interrupts in the NVIC
    NVIC_SetPriority(ADC0_IRQn, 1);

    //Enable ADC
    pac_adc_enable(1);
    // Start ADC sequencer
    pac_adc_start();
}

void config_adc_dtse_sequence_example_1(void)
{
    // Select PF4 as ADC Channel 4 Input
    PAC_GPIOF->MODE.P4 = 0;                                                 // PF4 = input
    PAC_SCC->PFMUXSEL.P4 = 0;                                               // Set Port F Pin 4 to ADC4 input

    // Select PF5 as ADC Channel 5 Input
    PAC_GPIOF->MODE.P5 = 0;                                                 // PF5 = input
    PAC_SCC->PFMUXSEL.P5 = 0;                                               // Set Port F Pin 5 to ADC5 input

    // Select PF6 as ADC Channel 6 Input
    PAC_GPIOF->MODE.P6 = 0;                                                 // PF6 = inputT
    PAC_SCC->PFMUXSEL.P6 = 0;                                               // Set Port F Pin 6 to ADC6 input

    //========================================================================================================
    // Setup DTSE Sequence Trigger - See "DTSE Trigger Mapping Table" in the PAC55XX Family User's Guide
    //========================================================================================================
    //  Timer C CCR5 = Trigger 21 and will initiate Sequence starting at Entry 0
    PAC_ADC->DTSETRIGENT20TO23.FORCE21 = 0;                                 // Reset to 0
    PAC_ADC->DTSETRIGENT20TO23.TRIG21CFGIDX = 0;                            // DTSE Trigger 0 Sequence Configuration Entry Index
    PAC_ADC->DTSETRIGENT20TO23.TRIG21EDGE = ADCDTSE_TRIGEDGE_RISING;        // PWMC5 rising edge

    // Convert ADC4, MUX = N/A;             EMUX => Don't Care
    pac_dtse_seq_config(0, ADC4, 0            , 0,        0);

    // Convert ADC5, MUX = N/A;             EMUX => Don't Care
    pac_dtse_seq_config(1, ADC5, 0            , 0,        0);

    // Convert ADC6, MUX = N/A;             EMUX => Don't Care
    pac_dtse_seq_config(2, ADC6, 0            , 0,        0);

    // Convert ADC0, MUX = AFEMUXOUT;       EMUX => VCORE
    pac_dtse_seq_config(3, ADC0, EMUX_VCORE   , 0,        0);

    // Convert ADC0, MUX = AFEMUXOUT;       EMUX => VCORE*4/10
    pac_dtse_seq_config(4, ADC0, EMUX_VCORE_DIV2P5, 0,    0);

    // Convert ADC0, MUX = AFEMUXOUT;       EMUX => VDDA*4/10
    pac_dtse_seq_config(5, ADC0, EMUX_VDDA_DIV2P5, 0,     0);

    // Convert ADC0, MUX = AFEMUXOUT;       EMUX => VCCIO*4/10
    pac_dtse_seq_config(6, ADC0, EMUX_VCCIO_DIV2P5, 0,    0);

    // Convert ADC0, MUX = AFEMUXOUT;       EMUX => VSYS*4/10
    pac_dtse_seq_config(7, ADC0, EMUX_VSYS_DIV2P5, 0,     0);

    // Convert ADC0, MUX = AFEMUXOUT;       EMUX => VPTAT
    pac_dtse_seq_config(8, ADC0, EMUX_VPTAT   , 0,        0);
    
    // Convert ADC0, MUX = AFEMUXOUT;       EMUX => VP/10
    pac_dtse_seq_config(9, ADC0, EMUX_VP_DIV10, 0,        0);
    
    // Convert ADC0, MUX = N/A;             EMUX => Don't care   End of Sequence
    pac_dtse_seq_config(10, ADC0, 0,  ADC_IRQ0_EN,  SEQ_END);

}


void init_adc_dtse_multi_sequence(void)
{
    // EMUX configuration
    pac_adc_config_emux_io();

    // Enable mux scan mode
    pac_tile_register_write(ADDR_AFEMUXCTL, 0x03);                            // BUFFEN=1, EMUX_EN=1, for auto-scan, sample and hold AO10SH=1, AO32SH=1, AO54SH=1

    // Configure ADC input pin PG7=ADO
    pac_adc_config_io(1);

    pac_adc_emux_config(ADCEMUXCTL_DTSE_SEQ, ADCEMUXCTL_EMUXDIV_DIV16);
    pac_adc_config(ADCCTL_MODE_DTSE, ADCCTL_CLKDIV_DIV16, 1);

    // Sequencer Configuration
    config_adc_dtse_multi_sequence();

    // Set ADC interrupt priority and enable ADC interrupt
    NVIC_EnableIRQ(ADC0_IRQn);                                                  // Enable ADC interrupts in the NVIC
    NVIC_SetPriority(ADC0_IRQn, 1);

    NVIC_EnableIRQ(ADC1_IRQn);                                                  // Enable ADC interrupts in the NVIC
    NVIC_SetPriority(ADC1_IRQn, 1);

    NVIC_EnableIRQ(ADC2_IRQn);                                                  // Enable ADC interrupts in the NVIC
    NVIC_SetPriority(ADC2_IRQn, 1);

    NVIC_EnableIRQ(ADC3_IRQn);                                                  // Enable ADC interrupts in the NVIC
    NVIC_SetPriority(ADC3_IRQn, 1);

    //Enable ADC
    pac_adc_enable(1);
    // Start ADC sequencer
    pac_adc_start();
}

void config_adc_dtse_multi_sequence(void)
{
    // Select PF4 as ADC Channel 4 Input
    PAC_GPIOF->MODE.P4 = 0;                                                 // PF4 = input
    PAC_SCC->PFMUXSEL.P4 = 0;                                               // Set Port F Pin 4 to ADC4 input

    // Select PF5 as ADC Channel 5 Input
    PAC_GPIOF->MODE.P5 = 0;                                                 // PF5 = input
    PAC_SCC->PFMUXSEL.P5 = 0;                                               // Set Port F Pin 5 to ADC5 input

    // Select PF6 as ADC Channel 6 Input
    PAC_GPIOF->MODE.P6 = 0;                                                 // PF6 = inputT
    PAC_SCC->PFMUXSEL.P6 = 0;                                               // Set Port F Pin 6 to ADC6 input
    
    // Select PF7 as ADC Channel 7 Input
    PAC_GPIOF->MODE.P7 = 0;                                                 // PF7 = inputT
    PAC_SCC->PFMUXSEL.P7 = 0;                                               // Set Port F Pin 7 to ADC7 input

    //========================================================================================================
    // Setup DTSE Sequence Triggers - See "DTSE Trigger Mapping Table" in the PAC55XX Family User's Guide
    //========================================================================================================
    //  Timer C CCR5 = Trigger 21 and will initiate Sequence starting at Entry 0
    PAC_ADC->DTSETRIGENT20TO23.FORCE21 = 0;									// Reset to 0
    PAC_ADC->DTSETRIGENT20TO23.TRIG21CFGIDX = 0;							// DTSE Trigger 0 Sequence Configuration Entry Index
    PAC_ADC->DTSETRIGENT20TO23.TRIG21EDGE = ADCDTSE_TRIGEDGE_RISING;        // PWMC5 rising edge

    //  Timer C CCR6 = Trigger 22 and will initiate Sequence starting at Entry 9
    PAC_ADC->DTSETRIGENT20TO23.FORCE22 = 0;									// Reset to 0
    PAC_ADC->DTSETRIGENT20TO23.TRIG22CFGIDX = 11;							// DTSE Trigger 9 Sequence Configuration Entry Index
    PAC_ADC->DTSETRIGENT20TO23.TRIG22EDGE = ADCDTSE_TRIGEDGE_RISING;        // PWMC6 rising edge

    //  Timer C CCR7 = Trigger 23 and will initiate Sequence starting at Entry 13
    PAC_ADC->DTSETRIGENT20TO23.FORCE23 = 0;									// Reset to 0
    PAC_ADC->DTSETRIGENT20TO23.TRIG23CFGIDX = 14;							// DTSE Trigger 13 Sequence Configuration Entry Index
    PAC_ADC->DTSETRIGENT20TO23.TRIG23EDGE = ADCDTSE_TRIGEDGE_RISING;        // PWMC7 rising edge

    //================================
    //===== Setup DTSE Sequences =====
    //================================

    //===== Setup DTSE Sequence; Convert VSYS, VP, VREF, VPTAT, VCORE; Starts at Entry 0 =====
    // Convert ADC0, MUX = AFEMUXOUT;   EMUX => VSYS*4/10
    pac_dtse_seq_config(0, ADC0, EMUX_VSYS_DIV2P5, 0,           0);

    // Dummy Entry                      EMUX => same as previous entry
    pac_dtse_seq_config(1, ADC0, EMUX_VSYS_DIV2P5, 0,           0);   // Conversion Not Used

    // Convert ADC0, MUX = AFEMUXOUT;   EMUX => VP/10
    pac_dtse_seq_config(2, ADC0, EMUX_VP_DIV10,    0,           0);

    // Dummy Entry                      EMUX => same as previous entry
    pac_dtse_seq_config(3, ADC0, EMUX_VP_DIV10,    0,           0);   // Conversion Not Used

    // Convert ADC0, MUX = AFEMUXOUT;   EMUX => VREF/2
    pac_dtse_seq_config(4, ADC0, EMUX_VREF_DIV2,   0,           0);

    // Dummy Entry                      EMUX => same as previous entry
    pac_dtse_seq_config(5, ADC0, EMUX_VREF_DIV2,   0,           0);   // Conversion Not Used
    
    // Convert ADC0, MUX = AFEMUXOUT;   EMUX => VPTAT
    pac_dtse_seq_config(6, ADC0, EMUX_VPTAT,       0,           0);

    // Dummy Entry                      EMUX => same as previous entry
    pac_dtse_seq_config(7, ADC0, EMUX_VPTAT,       0,           0);   // Conversion Not Used
    
    // Convert ADC0, MUX = AFEMUXOUT;   EMUX => VCORE
    pac_dtse_seq_config(8, ADC0, EMUX_VCORE      , 0,           0);

    // Dummy Entry                      EMUX => same as previous entry
    pac_dtse_seq_config(9, ADC0, EMUX_VCORE      , 0,           0);   // Conversion Not Used
    
    // Convert ADC0, MUX = N/A;         EMUX => Don't Care
    pac_dtse_seq_config(10, ADC0, 0     , ADC_IRQ0_EN,    SEQ_END);



    // ===== Setup DTSE Sequence; Convert ADC5, ADC6, ADC7; Starts at Entry 11 =====    
    // Convert ADC5, MUX = N/A;         EMUX => Don't Care
    pac_dtse_seq_config(11, ADC5, 0              , 0,           0);
    
    // Convert ADC6, MUX = N/A;         EMUX => Don't Care 
    pac_dtse_seq_config(12, ADC6, 0              , 0,           0);

    // Convert ADC7, MUX = N/A;         EMUX => Don't Care        Interrupt after Conversion, Sequence End
    pac_dtse_seq_config(13, ADC7, 0     , ADC_IRQ1_EN,    SEQ_END);



    //===== Setup DTSE Sequence; Convert VCORE, VCORE*4/10, VDDA*4/10, VCCIO*4/10, VREF/2; Starts at Entry 14 =====
    // Convert ADC0, MUX = AFEMUXOUT;   EMUX => VCORE*4/10
    pac_dtse_seq_config(14, ADC0, EMUX_VCORE_DIV2P5, 0,         0);

    // Dummy Entry                      EMUX => same as previous entry
    pac_dtse_seq_config(15, ADC0, EMUX_VCORE_DIV2P5, 0,         0);   // Conversion Not Used

    // Convert ADC0, MUX = AFEMUXOUT;   EMUX => VDDA*4/10
    pac_dtse_seq_config(16, ADC0, EMUX_VDDA_DIV2P5 , 0,         0);

    // Dummy Entry                      EMUX => same as previous entry
    pac_dtse_seq_config(17, ADC0, EMUX_VDDA_DIV2P5 , 0,         0);   // Conversion Not Used

    // Convert ADC0, MUX = AFEMUXOUT;   EMUX => VCCIO*4/10
    pac_dtse_seq_config(18, ADC0, EMUX_VCCIO_DIV2P5 , 0,        0);

    // Dummy Entry                      EMUX => same as previous entry
    pac_dtse_seq_config(19, ADC0, EMUX_VCCIO_DIV2P5 , 0,        0);   // Conversion Not Used

    // Convert ADC0, MUX = AFEMUXOUT;   EMUX => VCORE
    pac_dtse_seq_config(20, ADC0, EMUX_VCORE        , 0,        0);

    // Convert ADC0, MUX = AFEMUXOUT;   EMUX => VREF/2
    pac_dtse_seq_config(21, ADC0, EMUX_VREF_DIV2 , ADC_IRQ2_EN, 0);



    // Convert ADC0, MUX = N/A;         EMUX => Don't Care
    pac_dtse_seq_config(22, ADC0, 0                , 0,         0);

    // Convert ADC4, MUX = N/A;         EMUX => Don't Care      Interrupt after Conversion, Sequence end
    pac_dtse_seq_config(23, ADC4, 0    ,   ADC_IRQ3_EN,   SEQ_END);

}
