/**
 * @file bms_driver.h
 * @brief 
 */

//=============================================================================
// Copyright (C) 2023, Qorvo, Inc.
//
// THIS SOFTWARE IS SUBJECT TO A SOURCE CODE LICENSE AGREEMENT WHICH PROVIDES,
// AMONG OTHER THINGS:  (i) THAT IT CAN BE USED ONLY TO ADAPT THE LICENSEE'S
// APPLICATION TO PAC PROCESSORS SUPPLIED BY QORVO, INC.;
// (ii) THAT  IT IS PROVIDED "AS IS" WITHOUT WARRANTY;  (iii) THAT
// QORVO, INC. IS NOT LIABLE FOR ANY INDIRECT DAMAGES OR FOR DIRECT
// DAMAGES EXCEEDING US$1,500;  AND (iv) THAT IT CAN BE DISCLOSED TO AND USED
// ONLY BY CERTAIN AUTHORIZED PERSONS.
//
//=============================================================================

#ifndef BMS_DRIVER_H
#define BMS_DRIVER_H

#include "pac_cafe_arch1.h"
#include "pac_driver_tile.h"

//=============================================================================
//
// Primary Control and Status
//
//=============================================================================
/**
 * @brief This function to enable Signal Manager Tile
 * @param enabled Signal Manager Tile Enable. 0 = Disabled, 1 = Enabled. This bit is reset to 0 if nRST goes low.
 * @return none
 */
void afe_signal_manager_enable(uint32_t enabled);

/**
 * @brief This function to enable Charge pump
 * @param enabled Charge pump enable: 0 = disabled (default), 1 = enabled. 
 * @return none
 *
 */
void afe_charge_pump_enable(uint32_t enabled);

/**
 * @brief This function to enable Load detection
 * @param enabled Load detection enable: 0 = disabled, 1 = enabled,     
 * @return none
 * @note when enabled measure PACK+ to determine if a load is present
 */
void afe_load_detect_enable(uint32_t enabled);

/**
 * @brief This function to mux scpdac to vadc (BATOV channel)
 * @param enable scpdac2vadc: 1 = enabled, 0 = disabled.
 * @return none
 * @note N.A
 */
void afe_scpdac2vadc(uint32_t enable);

/**
 * @brief This function to overwrite CP_OK signal
 * @param val overwrite CP_OK: 1 = overwritten, 0 = not overwritten.
 * @return none
 * @note N.A
 */
void afe_dis_cpok_qual(uint32_t val);

/**
 * @brief This function to set MCU alive
 * @param alive MCU alive: 0 = dead, 1 = alive,     
 * @return none
 * @note For MCU to indicate AFE that it is alive. Before MCUALIVE is set, AFE ignore all MCU command except the SPI.
 */
void afe_set_mcu_alive(uint32_t alive);

/**
 * @brief This function to reset MCU 
 * @return none
 * @note Soft Reset. Write to "1" to assert nRST to generate system soft reset. TheSOFTRST is latched "1" so the MCU knows that after resumes from soft reset. This bit always read as "0".
 */
void afe_assert_nrst(void);

/**
 * @brief This function to config the HVBK switching frequency
 * @param frequency is the HVBK Switching Frequency Enumeration Type
 * @return none
 * @note HVBK Switching Frequency: 00b: 50kHz, 01b(default): 100kHz, 10b: 200kHz, 11b: 400KHz 
 */
void afe_hvbuck_set_switching_frequency(AFECTL2_HVBUCK_FREQSEL_Type frequency);

/**
 * @brief This function to set Drive Mode
 * @param mode is the Driver Mode Enumeration Type
 * @return none
 * @note Driver Mode is set to 00b if SEGEN = 0 or if nRST is active
 */
void afe_set_driver_mode(DRVCTL_DRVMODE_Type mode);

/**
 * @brief This function to enable AFEMUX and ADC buffer
 * @param emux_en EMUX Enable: 0 = disabled, 1 = enabled
 * @param buff_en ADC Buffer Circuitry Enable: 0 = disabled, 1 = enabled
 * @return none
 */
void afe_afemux_config(uint32_t emux_en, uint32_t buff_en);

/**
 * @brief This function to select AFEMUX
 * @param mux_type is the AFEMUX Selection Enumeration Type
 * @return none
 * @note These bits are writeable by SPI protocol when EMUX_EN=0 Bits should always be readable by SPI
 */
void afe_afemux_select_mux(AFEMUXSEL_AFEMUX_Type mux_type);

/**
 * @brief This function to config hibernate mode
 * @param pb_wake_en Push Button wake up enable: 0 = disabled, 1 = enabled
 * @param pack_wake_en PACK+ wake up enable: 0 = disabled, 1 = enabled
 * @param pack_wake_vref is the PACK Wakeup Vref Enumeration Type
 * @param wake_src is the Wakeup Source Enumeration Type
 * @param wake_timer is the Wakeup Timer Period Enumeration Type
 * @return none
 * @note if all wake enables are 0 when HIB is written to 1, then set PACKWAKEN and PBWAKEEN to 1. In hibernate output signals to the MCU must be shut off.
 */
void afe_hibernate_config(uint32_t pb_wake_en, uint32_t pack_wake_en, HIBCTL_PACKWAKEVREF_VOLSEL_Type pack_wake_vref, HIBCTL_WAKESRC_Type wake_src, HIBCTL_WAKETIMER_Type wake_timer);

/**
 * @brief This function to enter hibernate mode
 * @return none
 * @note Write HIB bit to enter hibernate. 0 = Operating; 1 = Hibernate Mode (this bit is cleared and power up sequence is initiated after the wake up event)
 */
void afe_hibernate_enter(void);

/**
 * @brief This function to get all reset status
 * @return Reset status. RSTSTAT[4] = PBRST, RSTSTAT[3] = HIBRST, RSTSTAT[2] = WDTRST, RSTSTAT[1] = SOFTRST, RSTSTAT[0] = FLTRST, 
 */
uint32_t afe_reset_get_status(void);

/**
 * @brief This function to clear reset status
 * @param status_mask is the value to clear status
 * @return none
 */
void afe_reset_clear_status(uint32_t status_mask);

/**
 * @brief This function to clear HWRST reset status
 */
void afe_reset_clear_fltrst(void);

/**
 * @brief This function to clear SOFT reset status
 */
void afe_reset_clear_softrst(void);

/**
 * @brief This function to clear Watchdog reset status
 */
void afe_reset_clear_wdtrst(void);

/**
 * @brief This function to clear Hibernate weakup reset status
 */
void afe_reset_clear_hibrst(void);

/**
 * @brief This function to clear Push Button Active for 8s reset status
 */
void afe_reset_clear_pbrst(void);

/**
 * @brief This function to config the push button
 * @param enabled Push Button Enable: 0 = disabled, 1 = enabled
 * @param reset_en Push Button Reset Enable: 0 = disabled, 1 = enabled. If enabled, PB active for 8 seconds shall reset the AFE.
 * @param deglitch_time is the Push Button Deglitch Time Enumeration Type
 * @param polarity is the Push Button Polarity Enumeration Type
 * @param pb_int_en Push Button Interrupt Enable: 0 = disabled, 1 = enabled
 * @return none
 */
void afe_pushbtn_config(uint32_t enabled, uint32_t reset_en, PB_DEGLITCH_TIMESEL_Type deglitch_time, PB_POLARITY_Type polarity, uint32_t pb_int_en);

/**
 * @brief This function to enable the push button
 * @param enabled Push Button Enable: 0 = disabled, 1 = enabled
 * @return none
 */
void afe_pushbtn_enable(uint32_t enabled);

/**
 * @brief This function to get PB interrupt flag
 * @return Push button interrupt status
 */
uint32_t afe_pushbtn_get_int_flag(void);

/**
 * @brief This function to clear PB interrupt flag
 * @return None
 */
void afe_pushbtn_clear_int_flag(void);

/**
 * @brief This function to get real time PB status
 * @return Push button status
 */
uint32_t afe_pushbtn_get_status(void);

/**
 * @brief This function to config AIO0
 * @param enabled AIO0 Buffer enable: 0 = disabled, 1 = enabled
 * @param mode is the AIO0 Mode Enumeration Type
 * @param swap_offset Swaps the Offset of the Buffer: 0 = disabled, 1 = enabled
 * @param mux_type is the AIO Mux Selection Enumeration Type
 * @return None
 */
void afe_aio0_config(uint32_t enabled, AIO0CFG_MODE_Type mode, uint32_t swap_offset, AIO0CFG_MUX_Type mux_type);

/**
 * @brief This function to config swap offset of AIO0
 * @param swap_offset Swaps the Offset of the Buffer: 0 = disabled, 1 = enabled
 * @return None
 */
void afe_aio0_swap_offset(uint32_t swap_offset);

/**
 * @brief This function to enable AIO0
 * @param enabled AIO0 Buffer enable: 0 = disabled, 1 = enabled
 * @return None
 */
void afe_aio0_enable(uint32_t enabled);

/**
 * @brief This function to enable the Battery Over Voltage DAC and Comparator
 * @param enabled Battery Over Voltage DAC and Comparator enable: 0 = disabled, 1 = enabled
 * @return None
 */
void afe_batov_daccomp_enable(uint32_t enabled);

/**
 * @brief This function to enable the Over Current Discharge DAC and Comparator 
 * @param enabled Over Current Discharge DAC and Comparator Enable: 0 = disabled, 1 = enabled
 * @return None
 */
void afe_ocd_daccomp_enable(uint32_t enabled);

/**
 * @brief This function to enable the Over Current Charge DAC and Comparator 
 * @param enabled Over Current Charge DAC and Comparator Enable: 0 = disabled, 1 = enabled
 * @return None
 */
void afe_occ_daccomp_enable(uint32_t enabled);

/**
 * @brief This function to enable the Short Circuit Protection DAC and Comparator 
 * @param enabled Short Circuit Protection DAC and Comparator Enable: 0 = disabled, 1 = enabled
 * @return None
 */
void afe_scp_daccomp_enable(uint32_t enabled);

/**
 * @brief This function to enable the I Sense Diff Amp 
 * @param enabled I Sense Diff Amp Enable: 0 = disabled, 1 = enabled
 * @return None
 */
void afe_isense_diffamp_enable(uint32_t enabled);

/**
 * @brief This function to enable the voltage ADC
 * @param enabled the voltage ADC Enable: 0 = disabled, 1 = enabled
 * @return None
 */
void afe_voltage_adc_enable(uint32_t enabled);

/**
 * @brief This function to enable the current ADC
 * @param enabled the current ADC Enable: 0 = disabled, 1 = enabled
 * @return None
 */
void afe_current_adc_enable(uint32_t enabled);

/**
 * @brief This function to enable PB Pass Through
 * @param enabled PB Pass Through Enable: 0 = disabled, 1 = enabled
 * @return None
 */
void afe_pushbtn_passthrough_enable(uint32_t enabled);

/**
 * @brief This function to enable the protection
 * @param batovd_prot_en Battery Over Voltage Protection Enable (1 = enabled, 0 = disabled) - Disables DSG FET
 * @param batovc_prot_en Battery Over Voltage Protection Enable (1 = enabled, 0 = disabled) - Disables CHG FET
 * @param ocd_prot_en Over Current Discharge Protection Enable (1 = enabled, 0 = diabled) - Disables DSG FET 
 * @param occ_prot_en Over Current Charge Protection Enable (1 = enabled, 0 = diabled) - Disables CHG FET
 * @param scpd_prot_en Short Circuit Protection Discharge FET Disable - Enable (1 = enabled, 0 = disabled) - Disables DSG FET
 * @param scpc_prot_en Short Circuit Protection Charge FET Disable - Enable (1 = enabled, 0 = disabled) - Disables CHG FET
 * @return None
 */
void afe_protection_enable(uint32_t batovd_prot_en, uint32_t batovc_prot_en, uint32_t ocd_prot_en, uint32_t occ_prot_en, uint32_t scpd_prot_en, uint32_t scpc_prot_en);

/**
 * @brief This function to enable the FUSE
 * @param enabled Enable FUSE FET gate drive output. 0: disabled (default); 1: enabled
 * @return None
 */
void afe_fuse_enable(uint32_t enabled);

/**
 * @brief This function to enable the power fault interrupt
 * @param hvcp_fault_en HVCP Fault Interrupt Enable (1 = enabled, 0 = disabled) 
 * @param vp_fault_en Battery Over Voltage Protection Enable (1 = enabled, 0 = disabled) 
 * @param vsys_fault_en Over Current Discharge Protection Enable (1 = enabled, 0 = disabled)  
 * @param vcore_fault_en Over Current Charge Protection Enable (1 = enabled, 0 = disabled) 
 * @param vdda_fault_en Short Circuit Protection Discharge FET Disable - Enable (1 = enabled, 0 = disabled) 
 * @param vccio_fault_en Short Circuit Protection Charge FET Disable - Enable (1 = enabled, 0 = disabled) 
 * @param drv_fault_en
 * @return None
 */
void afe_pwrfault_enable_int(uint32_t hvcp_fault_en, uint32_t vp_fault_en, uint32_t vsys_fault_en, uint32_t vcore_fault_en, uint32_t vdda_fault_en, uint32_t vccio_fault_en, uint32_t drv_fault_en);

/**
 * @brief This function to get the power fault status
 * @return Power fault status
 */
uint32_t afe_pwrfault_get_status(void);

/**
 * @brief This function to get the power fault status
 * @param fault_mask is the value to clear fault
 * @return None
 */
void afe_pwrfault_clear_status(uint32_t fault_mask);

/**
 * @brief This function to enable the temperature fault interrupt
 * @param twarn1_fault_en Temperature Warning 1 Fault Enable: 0 = disabled, 1 = enabled
 * @param twarn2_fault_en Temperature Warning 2 Fault Enable: 0 = disabled, 1 = enabled
 * @param tmp_fault_en Temperature Fault Enable: 0 = disabled, 1 = enabled
 * @param twarn1_cbd_en TWARN1 Cell Balance Enable: 0 = disabled, 1 = enabled. If enabled, then when a TWARN1 Fault occurs, all CB FETs will be disabled
 * @param twarn2_cbd_en TWARN2 Cell Balance Enable: 0 = disabled, 1 = enabled. If enabled, then when a TWARN2 Fault occurs, all CB FETs will be disabled
 * @return None
 */
void afe_tempfault_enable(uint32_t twarn1_fault_en, uint32_t twarn2_fault_en, uint32_t tmp_fault_en, uint32_t twarn1_cbd_en, uint32_t twarn2_cbd_en);

/**
 * @brief This function to get temperature fault status
 * @return Temperature fault status
 */
uint32_t afe_tempfault_get_status(void);

/**
 * @brief This function to clear temperature fault status
 * @param fault_mask is the value to clear temperature fault
 * @return None
 */
void afe_tempfault_clear_status(uint32_t fault_mask);

/**
 * @brief  This function sets enable state of Signal Fault interrupt
 * @param  batov_fault_en Battery Over Voltage Fault Enable: 0 = disabled, 1 = enabled
 * @param  ocd_fault_en Over Current Discharge Fault Enable: 0 = disabled, 1 = enabled
 * @param  occ_fault_en Over Current Charge Fault Enable: 0 = disabled, 1 = enabled
 * @param  scp_fault_en Short Circuit Protection Fault Enable: 0 = disabled, 1 = enabled
 * @param  emux_fault_en EMUX Fault Enable
 * @return none
 */
void afe_sigfault_enable(uint32_t batov_fault_en, uint32_t ocd_fault_en, uint32_t occ_fault_en, uint32_t scp_fault_en, uint32_t emux_fault_en);

/**
 * @brief  This function gets the status of Signal Faults
 * @return Status of Signal Faults
 */
uint32_t afe_sigfault_get_status(void);

/**
 * @brief  This function clears the Signal Faults
 * @param  fault_mask Mask of Signal Fault bits
 * @return none
 */
void afe_sigfault_clear_status(uint32_t fault_mask);

/**
 * @brief  This function gets the real time status of protection comparators
 * @return Real time status of protection comparators
 */
uint32_t afe_batrts_get_status(void);

//=============================================================================
//
// Battery Monitoring and Protection
//
//=============================================================================
/**
 * @brief  This function sets value of BATOV DAC
 * @param  value BATOV DAC value
 * @return none
 */
void afe_batov_set_dac(uint32_t value);

/**
 * @brief  This function configures BATOV Blanking Time
 * @param  time_base Blanking Time Base
 * @param  scale_factor Blanking Scale Factor
 * @return none
 */
void afe_batov_config(BATOVCFG_TIMEBASE_Type time_base, BATOVCFG_BLANKSF_Type scale_factor);

/**
 * @brief  This function configures the Voltage ADC and start the conversion if needed
 * @param  mux_type Voltage ADC Mux Selection
 * @param  start If set to a 1, start the Voltage ADC conversion
 * @return none
 */
void afe_vadc_config(VADCCTL_VBMUX_Type mux_type, uint32_t start);

/**
 * @brief  This function starts the Voltage ADC conversion
 * @return none
 */
void afe_vadc_start_conversion(void);

/**
 * @brief  This function gets the status of Voltage ADC busy flag
 * @return 1 if ADC conversion is active, 0 if ADC conversion is completed
 */
uint32_t afe_vadc_get_busy_flag(void);

/**
 * @brief  This function gets the result of the latest Voltage ADC conversion
 * @return Voltage ADC conversion result
 */
uint32_t afe_vadc_get_result(void);

/**
 * @brief  This function configures the Current ADC and start the conversion if needed
 * @param  da_gain Differential Amplifier Gain
 * @param  mux_type Current ADC Mux Selection
 * @param  start If set to a 1, start the Current ADC conversion
 * @return none
 */
void afe_iadc_config(IADCCTL_DAGAIN_Type da_gain, IADCCTL_IMUX_Type mux_type, uint32_t start);

/**
 * @brief  This function starts the Current ADC conversion
 * @return none
 */
void afe_iadc_start_conversion(void);

/**
 * @brief  This function gets the status of Current ADC busy flag
 * @return 1 if ADC conversion is active, 0 if ADC conversion is completed
 */
uint32_t afe_iadc_get_busy_flag(void);

/**
 * @brief  This function gets the result of the latest Current ADC conversion
 * @return Voltage ADC conversion result
 */
uint32_t afe_iadc_get_result(void);

//=============================================================================
//
// Short Circuit Protection
//
//=============================================================================
/**
 * @brief  This function sets value of SCP DAC
 * @param  value SCP DAC value
 * @return none
 */
void afe_scp_set_dac(uint32_t value);
/**
 * @brief  This function configures SCP Blanking Time
 * @param  time_base Blanking Time Base
 * @param  scale_factor Blanking Scale Factor
 * @return none
 */
void afe_scp_config(SCPCFG_TIMEBASE_Type time_base, SCPCFG_BLANKSF_Type scale_factor);

//=============================================================================
//
// Over Current Charge Protection
//
//=========================================-====================================
/**
 * @brief  This function sets value of OCC DAC
 * @param  value OCC DAC value
 * @return none
 */
void afe_occ_set_dac(uint32_t value);

/**
 * @brief  This function configures OCC Blanking Time
 * @param  time_base Blanking Time Base
 * @param  scale_factor Blanking Scale Factor
 * @return none
 */
void afe_occ_config(OCCCFG_TIMEBASE_Type time_base, OCCCFG_BLANKSF_Type scale_factor);

//=============================================================================
//
// Over Current Discharge Protection
//
//=========================================-====================================
/**
 * @brief  This function sets value of OCD DAC
 * @param  value OCD DAC value
 * @return none
 */
void afe_ocd_set_dac(uint32_t value);

/**
 * @brief  This function configures OCD Blanking Time
 * @param  time_base Blanking Time Base
 * @param  scale_factor Blanking Scale Factor
 * @return none
 */
void afe_ocd_config(OCDCFG_TIMEBASE_Type time_base, OCDCFG_BLANKSF_Type scale_factor);

//=============================================================================
//
// Cell Enablement / Monitoring
//
//=========================================-====================================
/**
 * @brief  This function sets the enable state of Cell Enable
 * @param  cell_bits If bit of index x is set to 1, enables the CE[x+1]. If set to 0, disables the CE[x+1]
 * @return none
 */
void afe_cell_enable(uint32_t cell_bits);

/**
 * @brief  This function sets the enable state of cell balance FET for VB
 * @param  cell_bits If bit of index x is set to 1, enables the VB[x+1] FET. If set to 0, disables the VB[x+1] FET
 * @return none
 */
void afe_cellbalance_enable_fet(uint32_t cell_bits);

//=============================================================================
//
// Miscellaneous
//
//=========================================-====================================
/**
 * @brief  This function sets the value of the USER register
 * @param  value The value written to the USER register
 * @return none
 */
void afe_gpreg_set(uint32_t value);

/**
 * @brief  This function gets value of the USER register
 * @return The value stored in the USER register
 */
uint32_t afe_gpreg_get(void);

/**
 * @brief  This function configures the Clock Output Circuit
 * @param  enabled If set to a 1, enables the Clock Output. If set to a 0, disables the Clock Output
 * @param  freq_select Clock Output Frequency Setting
 * @return none
 */
void afe_clkout_config(uint32_t enabled, CLKOUTCFG_FREQSEL_Type freq_select);

/**
 * @brief  This function sets the enable state of the Clock Output
 * @param  enabled If set to a 1, enables the Clock Output. If set to a 0, disables the Clock Output
 * @return none
 */
void afe_clkout_enable(uint32_t enabled);

/**
 * @brief  This function configures the WWDT
 * @param  enabled If set to a 1, enables the WWDT. If set to a 0, disables the WWDT
 * @param  clk_div WWDT clock divider
 * @param  cdv WWDT counter value
 * @param  window WWDT window value
 * @return none
 */
void afe_watchdog_config(uint32_t enabled, WWDTCTL_CLKDIV_Type clk_div, uint32_t cdv, uint32_t window);

/**
 * @brief  This function sets the enable state of the WWDT
 * @param  enabled If set to a 1, enables the WWDT. If set to a 0, disables the WWDT
 * @return none
 */
void afe_watchdog_enable(uint32_t enabled);

/**
 * @brief  This function gets the current WWDT counter value
 * @return Current WWDT counter value
 */
uint32_t afe_watchdog_get_ctr(void);

/**
 * @brief  This function resets the WWDT by reloading the CDV value into the CTR
 * @return none
 */
void afe_watchdog_reset(void);

void afe_iadc_restart_conversion(IADCCTL_DAGAIN_Type da_gain, IADCCTL_IMUX_Type mux_type);

void afe_vadc_restart_conversion(VADCCTL_VBMUX_Type cell_index);

#endif // #ifndef BMS_AFE_ARCH1_DRIVER_H
